/*******************************************
     sample code for attaching to and reading data from the CAVE
     tracker daemon 
********************************************/






/********
   Compile with:
	cc -I/usr/local/CAVE/include -L/usr/local/CAVE/include -o read-trackd read-trackd.c -lcave -lm
********/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <signal.h>
#include <limits.h>
#include <sys/prctl.h>
#include <sys/schedctl.h>
#include "trackd.h"

static int tracker_shmid=-1, controller_shmid=-1;
static struct TRACKD_TRACKING	*tracker=NULL;
static struct TRACKD_CONTROLLER	*controller=NULL;


/*************************************************************************
 void trackd_cleanup()
	When the program is interrupted by SIGINT, this routine closes the
 connections and exits.
**************************************************************************/
void trackd_cleanup(void)
{
 if (tracker)
	if (shmdt(tracker) < 0)
		perror("Detaching 'tracker' shared memory");
 if (controller)
	if (shmdt(controller) < 0)
		perror("Detaching 'controller' shared memory");
 exit(-1);
}

/*************************************************************************
 void get_tracker_mem()
	Attaches to shared memory for the sensors.  The global 'tracker'
  points to the chunk.
**************************************************************************/
void get_tracker_mem(CAVE_CONFIG_ST *config)
{
 tracker_shmid = shmget(config->TrackdKey, sizeof(struct TRACKD_TRACKING), 0);
 if (tracker_shmid < 0)
	{
	fprintf(stderr,"ERROR: can't get tracker shared memory\n");
	perror("shmget");
	return;
	}
 tracker = (struct TRACKD_TRACKING *) shmat(tracker_shmid,(void *) 0, 0);
 if (tracker == (struct TRACKD_TRACKING *) -1)
	{
	fprintf(stderr,"ERROR: can't attach tracker shared memory\n");
	perror("shmat");
	}
}

/*************************************************************************
 void get_controller_mem()
	Attaches to shared memory for the buttons/joystick.  The global
  'controller' points to the chunk.
**************************************************************************/
void get_controller_mem(CAVE_CONFIG_ST *config)
{
 controller_shmid = shmget(config->ControllerdKey, 
			sizeof(struct TRACKD_CONTROLLER), 0);
 if (controller_shmid < 0)
	{
	fprintf(stderr,"ERROR: can't get controller shared memory\n");
	perror("shmget");
	return;
	}
 controller = (struct TRACKD_CONTROLLER *) 
			shmat(controller_shmid,(void *) 0, 0);
 if (controller == (struct TRACKD_CONTROLLER *) -1)
	{
	fprintf(stderr,"ERROR: can't attach controller shared memory\n");
	perror("shmat");
	}
}

		

main(int argc,char **argv)
{
 CAVE_CONFIG_ST config;

 if (!getenv("CAVEDEBUGCONFIG"))
        putenv("CAVEDEBUGCONFIG=OFF");
 CAVEConfigurationInit(&argc,argv,NULL,&config);
	
 get_tracker_mem(&config);
 get_controller_mem(&config);

 signal(SIGINT,trackd_cleanup); /* catch interrupts to close nicely */

 while (1)
	{
	int i,j;
	if (tracker)
		{
		printf("%d sensors\n",tracker->header.numSensors);
		for (j=0; j < tracker->header.numSensors; j++)
			printf("sensor %d: (%.3f,%.3f,%.3f) (%.3f,%.3f,%.3f)\n", j,
				tracker->sensor[j].x,
				tracker->sensor[j].y,
				tracker->sensor[j].z,
				tracker->sensor[j].elev,
				tracker->sensor[j].azim,
				tracker->sensor[j].roll);
		}
	if (controller)
		{
		printf("controller:");
		printf(" %d buttons, %d valuators: ",controller->controller.num_buttons,controller->controller.num_valuators);
		for (j=0; j < controller->controller.num_buttons; j++)
			printf(" %d",controller->controller.button[j]);
		printf("   ");
		for (j=0; j < controller->controller.num_valuators; j++)
		printf(" %.3f",controller->controller.valuator[j]);
		printf("\n");
		}
	sleep(1);
	}
}
